import numpy as np
from numba import jit

def read_file(num, name):
    file_handle = open(name, mode='r')
    contents = file_handle.readlines()
    mean = np.zeros(num, dtype=float)
    std = np.zeros(num, dtype=float)
    corr = np.zeros((num,num), dtype=float)
    i = 0
    j = 0
    for each in contents:
        each = each.strip('\n').split(' ')
        if i==0:
            i += 1
            continue
        elif 0<i<=num:
            mean[j] = float(each[1])
            std[j] = float(each[2])
            i += 1
            j += 1
        elif i > num:
            m = int(each[1])-1
            n = int(each[2])-1
            corr[m,n]=float(each[3])
            corr[n,m]=float(each[3])
    cov = calculate_cov(num, corr, std)
    return mean, std, cov

def calculate_cov(num,corr, std):
    cov=np.zeros((num,num))
    for i in range(num):
        for j in range(num):
            cov[i,j] = corr[i,j]*std[i]*std[j]
    return cov

def generate_u(s2,num):
    rv = np.random.randn(1,s2)
    u_norm = np.linalg.norm(rv, keepdims=None)
    rv_norm = rv/u_norm
    index = np.random.choice(np.arange(num), size=s2, replace=False)
    u = np.zeros(num)
    u[index] = rv_norm
    return u

def generate_u_batch(s2, num, q):
    rv = np.random.randn(q,s2)
    u_norm = np.linalg.norm(rv, keepdims=True, axis=1)
    rv_norm = rv/u_norm
    return rv_norm

@jit(nopython=True, parallel=True)
def generate_u_batch_accelerated(s2, num, q):
    rv = np.random.randn(q,s2)
    u_norm = np.sqrt(np.sum(rv**2, axis=1)).reshape(-1, 1)
    rv_norm = rv/u_norm
    u = np.zeros((q, num))
    for i in range(q):
        index = np.random.choice(np.arange(num), size=s2, replace=False)
        u[i][index] = rv_norm[i]
    return u

def f(x, cov, mean, r, lam):
    y = np.exp(x)
    term_1 = np.dot(np.dot(y,cov), y.T)/(2*(np.sum(y)**2))
    a = np.sum(np.multiply(mean,y))/np.sum(y) - r
    term_2 =lam*(np.minimum(a,0)**2)
    func = term_1+ term_2
    return func
